import java.util.ArrayList;
import java.util.List;

import javafx.animation.Interpolator;
import javafx.animation.KeyFrame;
import javafx.animation.KeyValue;
import javafx.animation.Timeline;
import javafx.animation.TimelineBuilder;
import javafx.application.Application;
import javafx.beans.binding.Bindings;
import javafx.beans.property.IntegerProperty;
import javafx.beans.property.SimpleIntegerProperty;
import javafx.beans.value.ChangeListener;
import javafx.beans.value.ObservableValue;
import javafx.event.ActionEvent;
import javafx.event.EventHandler;
import javafx.scene.Group;
import javafx.scene.GroupBuilder;
import javafx.scene.Scene;
import javafx.scene.SceneBuilder;
import javafx.scene.control.Button;
import javafx.scene.control.Label;
import javafx.scene.input.MouseEvent;
import javafx.scene.shape.Circle;
import javafx.scene.text.Font;
import javafx.stage.Stage;
import javafx.util.Duration;

public class Hunter extends Application {
	private static final int INITIAL_SCORE = 0;
	private static final int INITIAL_LIVES = 3;
	private final IntegerProperty score = new SimpleIntegerProperty(INITIAL_SCORE);
	private final IntegerProperty lives = new SimpleIntegerProperty(INITIAL_LIVES);
	private final List<Timeline> moveTimelines = new ArrayList<>();
	private Group enemyGroup;
	private Scene scene;
	private Timeline createEnemyTimeline;

	public static void main(String[] args) {
		Application.launch(args);
	}

	@Override
	public void start(Stage primaryStage) throws Exception {
		primaryStage.setTitle("Hunter");

		enemyGroup = new Group();

		Label information = new Label();
		information.setFont(new Font(24));
		information.textProperty().bind(
				Bindings.format("Score: %s, Lives: %s", score, lives));

		Button restartButton = new Button("Restart");

		scene = SceneBuilder
				.create()
				.width(600)
				.height(400)
				.root(GroupBuilder.create()
						.children(information, enemyGroup, restartButton)
						.build()).build();

		restartButton.translateXProperty().bind(
				scene.widthProperty().subtract(restartButton.widthProperty())
						.divide(2));

		restartButton.translateYProperty().bind(
				scene.heightProperty().subtract(restartButton.heightProperty())
						.divide(2));
		
		restartButton.visibleProperty().bind(lives.isEqualTo(0));
		
		restartButton.setOnAction(new EventHandler<ActionEvent>() {
			@Override
			public void handle(ActionEvent event) {
				restartGame();
			}
		});

		createEnemyTimeline = TimelineBuilder
				.create()
				.cycleCount(Timeline.INDEFINITE)
				.keyFrames(
						new KeyFrame(Duration.seconds(1),
								new EventHandler<ActionEvent>() {
									@Override
									public void handle(ActionEvent event) {
										createEnemy();
									}
								})).build();

		createEnemyTimeline.play();

		lives.addListener(new ChangeListener<Number>() {
			@Override
			public void changed(ObservableValue<? extends Number> observable,
					Number oldValue, Number newValue) {
				if (newValue.intValue() == 0) {
					stopGame();
				}
			}
		});

		primaryStage.setScene(scene);

		primaryStage.show();
	}

	private void createEnemy() {
		final Circle circle = new Circle(0, Math.random() * scene.getHeight(),
				20);

		circle.setOnMousePressed(new EventHandler<MouseEvent>() {
			@Override
			public void handle(MouseEvent event) {
				if (lives.get() > 0) {
					enemyGroup.getChildren().remove(circle);
					score.set(score.get() + 1);
				}
			}
		});

		enemyGroup.getChildren().add(circle);

		Timeline move = new Timeline();
		move.getKeyFrames().add(
				new KeyFrame(Duration.seconds(3),
						new EventHandler<ActionEvent>() {
							@Override
							public void handle(ActionEvent event) {
								if (enemyGroup.getChildren().remove(circle)) {
									lives.set(lives.get() - 1);
								}
							}
						}, new KeyValue(circle.centerXProperty(), scene
								.getWidth(), Interpolator.EASE_BOTH)));
		move.play();

		moveTimelines.add(move);

	}

	private void restartGame() {
		enemyGroup.getChildren().clear();
		lives.set(INITIAL_LIVES);
		score.set(INITIAL_SCORE);
		createEnemyTimeline.play();
	}
	
	private void stopGame() {
		createEnemyTimeline.stop();
		for (Timeline move : moveTimelines) {
			move.pause();
		}
		moveTimelines.clear();
	}
}
